package com.ziplinegreen.vault.Controller;

import com.sun.deploy.net.HttpResponse;
import com.ziplinegreen.vault.Exception.ResourceNotFoundException;
import com.ziplinegreen.vault.Model.Post;
import com.ziplinegreen.vault.Model.User;
import com.ziplinegreen.vault.Repository.PostRepository;
import com.ziplinegreen.vault.Repository.UsersRepository;
import com.ziplinegreen.vault.Service.PostService;
import com.ziplinegreen.vault.Service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.messaging.handler.annotation.DestinationVariable;
import org.springframework.messaging.handler.annotation.MessageMapping;
import org.springframework.messaging.handler.annotation.SendTo;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Set;

@RestController
public class PostController {

    private PostService postService;
    private UserController userController;
    private PostRepository repository;

    @Autowired
    public PostController(PostService postService, UserController userController, PostRepository repository) {
        this.postService = postService;
        this.userController = userController;
        this.repository = repository;
    }

    @PostMapping("/posts")
    public ResponseEntity createPost(@RequestBody Post post) {
        return new ResponseEntity(repository.save(post), HttpStatus.CREATED);

    }


    @MessageMapping("/posts/{userId}")
    @SendTo("/topic/posts")
    public Post createPost(@DestinationVariable("userId") Long userId, @RequestBody Post post) {
        post.setUserName(userController.getUserById(userId).getUsername());
        return repository.save(post);

    }

    @GetMapping("/posts/all")
    public Iterable<Post> getAllPosts() {
        return repository.findAll();
    }



    @GetMapping("/posts/{userId}")
    public ResponseEntity<Iterable<Post>> getAllPostsByUserId(@PathVariable Long userId) {
        return postService.findByUserId(userId);
    }

//    @PutMapping("/users/{userId/posts")
//    public ResponseEntity<Post> updatePost(@PathVariable Long userId, @RequestBody Post post){
//        return postService.updatePost(userId, post);
//    }



//    @PutMapping("/users/{userId}/posts/{postId}")
//    public Post updateComment(@PathVariable (value = "userId") Long userId,
//                                 @PathVariable (value = "postId") Long postId,
//                                 @Valid @RequestBody Post postRequest) {
//        if(!userRepository.existsById(userId)) {
//            throw new ResourceNotFoundException("UserId " + userId + " not found");
//        }
//
//        return postRepository.findById(postId).map(post -> {
//            post.setMessage(postRequest.getMessage());
//            return postRepository.save(post);
//        }).orElseThrow(() -> new ResourceNotFoundException("PostId " + postId + "not found"));
//    }
//
//    @DeleteMapping("/posts/{postId}/comments/{commentId}")
//    public ResponseEntity<?> deleteComment(@PathVariable (value = "userId") Long userId,
//                                           @PathVariable (value = "postId") Long postId) {
//        if(!userRepository.existsById(userId)) {
//            throw new ResourceNotFoundException("UserId " + userId + " not found");
//        }
//
//        return postRepository.findById(postId).map(post -> {
//            postRepository.delete(post);
//            return ResponseEntity.ok().build();
//        }).orElseThrow(() -> new ResourceNotFoundException("PostId " + postId + " not found"));
//    }
}
